
# q-expansion of newform 8470.2.a.v, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8470
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [-2], [1], [1], [0], [0], [0], [0], [-4], [-2], [-2], [-6], [-8], [12], [-6], [-6], [-10], [4], [-8], [-4], [4], [16], [0], [-6], [14], [0], [-14], [-12], [6], [2], [-8], [-4], [6], [-8], [-6], [8], [6], [-8], [-16], [-24], [12], [-6], [-8], [22], [-14], [-6], [12], [-6], [-4], [2], [18], [0], [16], [10], [-14], [24], [10], [-20], [2], [2], [-28], [4], [-28], [-6], [2], [-2], [28], [-14], [-28], [20], [18], [8], [-22], [-14], [-16], [-2], [22], [-34], [-6], [-4], [-26], [18], [-8], [14], [-4], [-20], [-10], [-26], [20], [4], [-30], [-20], [20], [-36], [-40], [-12], [18], [-34], [-28], [10], [44], [18], [-32], [-6], [-20], [-26], [2], [12], [20], [8], [-24], [6], [-42], [-22], [-8], [46], [34], [-42], [22], [20], [46], [-14], [0], [20], [6], [14], [30], [-34], [14], [-4], [-12], [8], [52], [-26], [-12], [-12], [18], [-48], [6], [-18], [44], [34], [32], [36], [-10], [-10], [8], [-12], [2], [-32], [-10], [-26], [-44], [52], [48], [28], [8], [30], [-36], [60], [-12], [6], [48], [26], [42], [-22], [-4], [48], [46], [-54], [-44], [14], [24], [34], [-54], [22], [20], [-6], [-20], [-8], [32], [48], [54], [-24], [16], [18], [42], [34], [-14], [56], [38], [4], [64], [-10], [-12], [-6], [-30], [-30], [44], [-44], [2], [0], [-62], [60], [66], [-66], [10], [-44], [-42], [-10], [2], [-22], [-48], [62], [36], [26], [-72], [-52], [-44], [54], [6], [52], [30], [-8], [52], [-26], [-34], [48], [-28], [-60], [-46], [20], [-56], [46], [58], [-20], [-2], [-22], [-16], [-26], [48], [16], [-42], [42], [0], [0], [-2], [-2], [20], [24], [42], [-60], [-60], [-14], [28], [20], [10], [-80], [-34], [32], [-56], [-12], [52], [64], [42], [66], [66], [62], [-52], [46], [70], [64], [-18], [-48], [-20], [2], [4], [42], [-2], [0], [-16], [36], [52], [0], [-16], [28], [-50], [-22], [-62], [48], [-6], [58], [42], [32], [34], [-12], [54], [74], [-32], [8], [-28], [86], [72], [48], [88], [-66], [12], [78], [-80], [-22], [-40], [-10], [18], [-76], [64], [4], [-54], [-72], [-10], [-54], [92], [-78], [60], [48], [30], [-14], [56], [-46], [32], [14], [10], [-78], [0], [-32], [36], [-78], [-62], [-8], [10], [52], [-74], [-26], [-88], [-40], [-90], [36], [-6], [-8], [-72], [-6], [-40], [62], [-2], [20], [26], [0], [-78], [52], [46], [-24], [-22], [-88], [48], [-60], [-30], [6], [-40], [8], [78], [-56], [-50], [88], [-46], [24], [-54], [18], [-40], [32], [-8], [-24], [78], [78], [-96], [-10], [16], [-70], [-80], [38], [-12], [-30], [-66], [50], [8], [10], [0], [-54], [-84], [98], [-96], [-90], [-36], [-62], [32], [-78], [-80], [64], [14], [78], [-18], [30], [44], [-42], [76], [52], [-62], [-18], [64], [-42], [66], [-48], [6], [52], [-12], [84], [58], [10], [-40], [10], [-74], [-36], [50], [-60], [-28], [-86], [102], [70], [-6], [-74], [-64], [-32], [18], [30], [46], [32], [-20], [22], [88], [-56], [-26]]
